/*
 * at89-prog - spi.c
 *
 * 	SPI Master
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <linux/ppdev.h>

#include <prog-io.h>
#include <at89-prog.h>
#include <spi.h>

static unsigned long spi_sleep;
static int clock_polarity;

#include <time.h>
static unsigned long long time_ns(void)
{
	struct timespec ts;

	clock_gettime(CLOCK_MONOTONIC, &ts);
	return ts.tv_sec * 1000000000ULL + ts.tv_nsec;
}

void my_nsleep(unsigned long ns)
{
	unsigned long long t;

	t = time_ns();
	while (time_ns() - t < ns);
}

void spi_send(uint8_t x)
{
	spi_xmit(x);
}

uint8_t spi_recv(void)
{
	return spi_xmit(0);
}

uint8_t spi_xmit(uint8_t x)
{
	uint8_t r = 0;
	int i;

	for (i = 7; i >= 0; i--) {
		/* TODO: optimize this, it can be done in one op */
		pin_set(PIN_SCK, clock_polarity);
		pin_set(PIN_MOSI, x & (1 << i));
		my_nsleep(spi_sleep);
		r |= pin_get(PIN_MISO) << i;
		pin_set(PIN_SCK, !clock_polarity);
		my_nsleep(spi_sleep);
	}
	return r;
}

void spi_init(unsigned long spi_clock, int polarity)
{
	pin_set(PIN_SCK, 0);

	spi_sleep = 1000000000 / (2 * spi_clock);
	if (verbose_level > 0)
		fprintf(stderr, "spi: using %ldns delay\n", spi_sleep);
	clock_polarity = polarity;
//	spi_sleep = 0;
}
